﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace BMM 
{
    public class BarretBehaviour : EnhancedLimb
    {
        BallisticsEmitter Ballistics;
        Transform muzzleTransform;
        ParticleSystem flash;
        AudioClip[] ShotSounds;
        AudioSource audioSource;
        GameObject BaseGun;


        void Awake()
        {
            base.Initialize(ModResources.barretLimb);

            BaseGun = ModAPI.FindSpawnable("Submachine Gun").Prefab;
            Ballistics = new BallisticsEmitter((MonoBehaviour)this, ModAPI.FindCartridge("7.62x51mm"));
            Ballistics.LayersToHit = BaseGun.GetComponent<FirearmBehaviour>().LayersToHit;
            muzzleTransform = UnityEngine.Object.Instantiate<GameObject>(Resources.Load<GameObject>("Prefabs/MuzzleSmoke"), transform).transform;
            flash = muzzleTransform.GetComponentInChildren<ParticleSystem>();
            ShotSounds = BaseGun.GetComponent<FirearmBehaviour>().ShotSounds;
            audioSource = gameObject.AddComponent<AudioSource>();
            audioSource.volume = 1;
            audioSource.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            audioSource.outputAudioMixerGroup = Global.main.SoundEffects;

            var barrel = gameObject.AddComponent<BoxCollider2D>();
            barrel.offset = new Vector2(0.015f, -0.3f);
            barrel.size = new Vector2(0.05f, 0.15f);

            var allColliders = gameObject.transform.root.GetComponentsInChildren<Collider2D>();
            foreach (var a in allColliders)
                foreach (var b in allColliders)
                    Physics2D.IgnoreCollision(a, b);
        }

        void Shoot()
        {
            if (UserPreferenceManager.Current.TracerBullets)
            {
                Ballistics.MaxRange = Mathf.Clamp(this.BaseGun.GetComponent<FirearmBehaviour>().Cartridge.StartSpeed / 1f, 1f, 1f);
                Ballistics.DoTraversal = true;
            }
            else
            {
                Ballistics.MaxRange = 1000f;
                Ballistics.DoTraversal = false;
            }

            Ballistics.Emit(transform.position + -transform.up * 0.35f,-transform.up);
            ExplosionCreator.CreatePulseExplosion(transform.position, Ballistics.Cartridge.Recoil / 100f, Ballistics.Cartridge.Recoil / 1f, false, false);
            CameraShakeBehaviour.main.Shake((float)((double)Ballistics.Cartridge.ImpactForce * 0.8000000225678 * (double)BaseGun.GetComponent<FirearmBehaviour>().BulletsPerShot / 0.89999989999), transform.position);
            muzzleTransform.localScale = Vector3.one * Mathf.Clamp(Mathf.Pow(Ballistics.Cartridge.Damage * (float)BaseGun.GetComponent<FirearmBehaviour>().BulletsPerShot, 0.5f) / 3, 0.22f, 2.111f);
            muzzleTransform.localScale = new Vector3(this.transform.lossyScale.x * muzzleTransform.localScale.x, muzzleTransform.localScale.y, muzzleTransform.localScale.z);
            muzzleTransform.position = transform.position + -transform.up * 0.35f;
            muzzleTransform.right = transform.position + -transform.up * 0.35f;
            flash.Play(true);
            audioSource.PlayOneShot(((IList<AudioClip>)ShotSounds).PickRandom<AudioClip>());
        }

        void Use()
        {
            StartCoroutine(Burst());
        }

        IEnumerator Burst()
        {
            int i = 0;
            while (i <= 3)
            {
                yield return new WaitForSeconds(0.075f);

                Shoot();

                i++;
            }
        }
    }
}