using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;

namespace BMM
{
    #region SyringeObjects
    internal class LaserEyeSyringe : OldSyringeBehaviour
    {
        public override Type GetPoisonType()
        {
            return typeof(LaserEyeExamplePoison);
        }
    }
    internal class HeadExploderExampleSyringe : OldSyringeBehaviour
    {
        public override Type GetPoisonType()
        {
            return typeof(HeadExploderExamplePoison);
        }
    }

    internal class HealerExampleSyringe : OldSyringeBehaviour
    {
        public override Type GetPoisonType()
        {
            return typeof(HealerExamplePoison);
        }
    }
    internal class MysterioSyringe : OldSyringeBehaviour
    {
        public override Type GetPoisonType()
        {
            return typeof(Mysteriopoison);
        }
    }

    internal class electricHandsSyringe : OldSyringeBehaviour
    {
        public override Type GetPoisonType()
        {
            return typeof(Electricpoison);
        }
    }
    #endregion


    public class HulkPoison : PoisonSpreadBehaviour
    {
        public LimbBehaviour limb;

		public override float SpreadSpeed
		{
			get
			{
				return Mathf.Infinity;
			}
		}

        public override float Lifespan
        {
            get
            {
                return 1;
            }
        }

        public override void Start()
        {
            PhysicalBehaviour physicalBehaviour = this.Limb.PhysicalBehaviour;

                                Limb.BreakingThreshold *= 10f;
                                Limb.ImmuneToDamage = true;
                                Limb.ImpactPainMultiplier = 0f;
                                Limb.ShotDamageMultiplier = 0f;
                                Limb.Vitality *= 100f;
                                Limb.Health *= 100f;
                                Limb.FakeUprightForce *= 1.65f;
                                Limb.InitialHealth *= 100f;
                                Limb.IsLethalToBreak = false;
                                Limb.PhysicalBehaviour.BurningProgressionMultiplier = 0f;
                                Limb.CirculationBehaviour.BloodLossRateMultiplier = 0f;
                                Limb.CirculationBehaviour.ImmuneToDamage = true;
			                    Limb.GetComponent<SpriteRenderer>().color = Color.green;
                                Limb.transform.localScale = new Vector2(((Limb.transform.localScale.x <= 0)? -1.5f : 1.5f), 1.5f);

        }

		void Update()
        {
            Limb.CirculationBehaviour.HealBleeding();
            Limb.Person.Consciousness = 1f;
        }
    }


    public class LaserEyeExamplePoison : PoisonSpreadBehaviour
    {
        public LimbBehaviour limb;

        public override float SpreadSpeed
        {
            get
            {
                return 0;
            }
        }

        public override float Lifespan
        {
            get
            {
                return 1;
            }
        }

        public override void Start()
        {
            PhysicalBehaviour physicalBehaviour = this.Limb.PhysicalBehaviour;
            foreach (var body in physicalBehaviour.GetComponent<LimbBehaviour>().Person.Limbs)
            {
                if (body.gameObject.name.Contains("Head"))
                    LaserEyesPowerExample.SetUpPower(body, physicalBehaviour.GetComponent<LimbBehaviour>().Person, body.Person.gameObject, Color.red);
            }
        }
    }

    public class HeadExploderExamplePoison : PoisonSpreadBehaviour
    {
        public LimbBehaviour limb;

        public override float SpreadSpeed
        {
            get
            {
                return 0;
            }
        }

        public override float Lifespan
        {
            get
            {
                return 1;
            }
        }

        public override void Start()
        {
            PhysicalBehaviour physicalBehaviour = this.Limb.PhysicalBehaviour;
            foreach (var body in physicalBehaviour.GetComponent<LimbBehaviour>().Person.Limbs)
            {
                if (body.gameObject.name.Contains("Head"))
                    HeadExploderPowerExample.SetUpPower(body, physicalBehaviour.GetComponent<LimbBehaviour>().Person, body.Person.gameObject);
            }
        }



    }
	class LaserPoison : PoisonSpreadBehaviour
	{
		public override float SpreadSpeed
		{
			get
			{
				return Mathf.Infinity;
			}
		}

        public override void Start()
		{
            Limb.gameObject.AddComponent<LaserBehaviour>();
		}

		public LaserPoison()
		{
		}
	}

    class LaserSyringe : OldSyringeBehaviour
	{
		public override Type GetPoisonType()
		{
			return typeof(LaserPoison);
		}
	}
    class LaserBehaviour : MonoBehaviour
    {
		LimbBehaviour Limb;

		void Awake()
        {
			Limb = gameObject.GetComponent<LimbBehaviour>();
            if (Limb.GetComponent<GripBehaviour>())
            {
                Limb.gameObject.AddComponent<UseEventTrigger>().Action = () =>
                {
                    GameObject HandOBJ = UnityEngine.Object.Instantiate(ModAPI.FindSpawnable("Blaster Rifle").Prefab.GetComponent<BlasterBehaviour>().Bolt);
                                    
                    ModAPI.CreateParticleEffect("Vapor", Limb.transform.position);

                    HandOBJ.GetComponent<BlasterboltBehaviour>().Trail.startColor = Color.red;
                    HandOBJ.GetComponent<BlasterboltBehaviour>().Trail.endColor = Color.red;
                    HandOBJ.GetComponent<BlasterboltBehaviour>().Trail.widthMultiplier += 0.1f;
                                    
                    HandOBJ.transform.position = Limb.gameObject.transform.position;
                    HandOBJ.transform.eulerAngles = Limb.gameObject.transform.eulerAngles;
                    HandOBJ.transform.eulerAngles += new Vector3(0, 0, 270);

					AudioClip Repulsor = ModAPI.LoadSound("People/Tony/beam.mp3"); 

                    AudioSource audio = Limb.gameObject.AddComponent<AudioSource>();
                    audio.spatialBlend = 1;
                    audio.PlayOneShot(Repulsor);
                    };
                }
                    
            }

    }

    public class HealerExamplePoison : PoisonSpreadBehaviour
    {
        public LimbBehaviour limb;

        public override float SpreadSpeed
        {
            get
            {
                return 0;
            }
        }

        public override float Lifespan
        {
            get
            {
                return 1;
            }
        }

        public override void Start()
        {
            PhysicalBehaviour physicalBehaviour = this.Limb.PhysicalBehaviour;
            foreach (var body in physicalBehaviour.GetComponent<LimbBehaviour>().Person.Limbs)
            {
                if (body.gameObject.name.Contains("UpperBody"))
                    HealerExamplePower.SetUpPower(body, physicalBehaviour.GetComponent<LimbBehaviour>().Person, body.Person.gameObject);
            }
        }



    }
    public class Mysteriopoison : PoisonSpreadBehaviour
    {
        public LimbBehaviour limb;

        public override float SpreadSpeed
        {
            get
            {
                return 0;
            }
        }

        public override float Lifespan
        {
            get
            {
                return 1;
            }
        }

        public override void Start()
        {
            PhysicalBehaviour physicalBehaviour = this.Limb.PhysicalBehaviour;
            foreach (var body in physicalBehaviour.GetComponent<LimbBehaviour>().Person.Limbs)
            {
                if (body.gameObject.name.Contains("LowerArm"))
                    Mysteriopower.SetUpPower(body, physicalBehaviour.GetComponent<LimbBehaviour>().Person, body.Person.gameObject);
            }
        }



    }
    public class Electricpoison : PoisonSpreadBehaviour
    {
        public LimbBehaviour limb;

        public override float SpreadSpeed
        {
            get
            {
                return 0;
            }
        }

        public override float Lifespan
        {
            get
            {
                return 1;
            }
        }

        public override void Start()
        {
            PhysicalBehaviour physicalBehaviour = this.Limb.PhysicalBehaviour;
            foreach (var body in physicalBehaviour.GetComponent<LimbBehaviour>().Person.Limbs)
            {
                if (body.gameObject.name.Contains("LowerArm"))
                    Electropower.SetUpPower(body, physicalBehaviour.GetComponent<LimbBehaviour>().Person, body.Person.gameObject);
            }
        }



    }
    public class GrenadierExamplePoison : PoisonSpreadBehaviour
    {
        public LimbBehaviour limb;

        public override float SpreadSpeed
        {
            get
            {
                return 0;
            }
        }

        public override float Lifespan
        {
            get
            {
                return 1;
            }
        }

        public override void Start()
        {
            PhysicalBehaviour physicalBehaviour = this.Limb.PhysicalBehaviour;
            foreach (var body in physicalBehaviour.GetComponent<LimbBehaviour>().Person.Limbs)
            {
                if (body.gameObject.name.Contains("LowerArm"))
                    GrenadierPowerExample.SetUpPower(body, physicalBehaviour.GetComponent<LimbBehaviour>().Person, body.Person.gameObject);
            }
        }



    }







}    
